#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : zb
# @Time    : 2020/11/14 16:37
# @Blog    : https://blog.zbmain.com

import hashlib
import http.client
import json
import random
import urllib
from urllib.request import urlopen
__all__ = ['__getBaiduTranslateConfigFromJson','BaiduTranslate','YoudaoTranslate','GoogleTranslate']

def __getBaiduTranslateConfigFromJson(configUrl=''):
    '''
    json格式：
    {
        "translate":{
            "baidu":{
                "appid":"",
                "secretKey":""
            },
            "google":{
                "appid":"",
                "secretKey":""
            },"youdao":{
                "appid":"",
                "secretKey":""
            }
        }
    }
    :param configUrl:
    :return:
    '''
    configJSON = configUrl or 'https://zbmain.com/files/others/config.json'
    resultJSON = json.loads(urlopen(configJSON).read())
    return resultJSON['translate']['baidu']['appid'], resultJSON['translate']['baidu']['secretKey']


class BaiduTranslate():
    def __init__(self, appid, secretKey, fromLang='en', toLang='cn', apiUrl=''):
        '''
        appid、secretKey自行前往官方注册
        :param appid:
        :param secretKey:
        :param fromLang: 翻译器的源语种，默认英文
        :param toLang: 翻译器的目标语种，默默中文
        :param apiUrl: api地址，默认空，若官方更新接口可新设置
        '''
        self.apiUrl = apiUrl or '/api/trans/vip/translate'
        self.appid = appid
        self.secretKey = secretKey
        self.fromLang = fromLang
        self.toLang = toLang

    def __call__(self, text, fromLang='', toLang=''):
        '''
        :param text: 翻译输入
        :param fromLang: 临时源语种【可选】
        :param toLang: 临时目标语种【可选】
        :return: (是否成功,输出,输入)
        '''
        fromLang = fromLang or self.fromLang
        toLang = toLang or self.toLang
        salt = str(random.randint(32768, 65536))
        sign = self.appid + text + salt + self.secretKey
        sign = hashlib.md5(sign.encode(encoding='utf-8')).hexdigest()

        requestUrl = self.apiUrl + '?appid=' + self.appid + '&q=' + urllib.parse.quote(
            text) + '&from=' + fromLang + '&to=' + toLang + '&salt=' + salt + '&sign=' + sign

        try:
            httpClient = http.client.HTTPConnection('api.fanyi.baidu.com')
            httpClient.request('GET', requestUrl)

            # response是HTTPResponse对象
            response = httpClient.getresponse()
            result_all = response.read().decode("utf-8")
            result = json.loads(result_all)
            return True, result["trans_result"][0]["dst"], result["trans_result"][0]["src"]
        except Exception as e:
            return False, e
        finally:
            if httpClient:
                httpClient.close()


class YoudaoTranslate():
    def __init__(self):
        print('To be updated')


class GoogleTranslate():
    def __init__(self):
        print('To be updated')


if __name__ == "__main__":
    appid, secretKey = __getBaiduTranslateConfigFromJson()
    baiduTranslate = BaiduTranslate(appid, secretKey, 'auto', 'en')
    print(baiduTranslate('你好，世界！'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : zb
# @Time    : 2020/11/18 00:50
# @Blog    : https://blog.zbmain.com

import time as t


def cuttentDateTime():
    '''2020-01-01 00:00:00'''
    return t.strftime("%Y-%m-%d %H:%M:%S")


def cuttentTime():
    '''00:00:00'''
    return t.strftime("%H:%M:%S")


# 作用：减少主函数import time
def time():
    return t.time()


def timeSince(since):
    '''
    阶段耗时
    :param since: 开始时间
    :return: 开始 >> 此刻
    '''
    s = t.time() - since
    m = int(s / 60)
    s -= m * 60
    return '%dm %ds' % (m, s)


if __name__ == "__main__":
    since = t.time() - 9.9 * 60
    print(timeSince(since))
    t.sleep(5)
    print(cuttentTime())
    print(t.strftime("%Y-%m-%d %H:%M:%S"))
    print(cuttentDateTime())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : zb
# @Time    : 2020/11/15 20:46
# @Blog    : https://blog.zbmain.com
# 个人特殊使用的代码模块使用： __命名（[__类名]（类中变量|函数不需要__））
# __all__ = []
__version__ = '0.0.5'
__author__ = 'zhaobin'
__install__ = 'pip install zbmain'
'''pip install zbmain'''



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : zb
# @Time    : 2020/11/18 01:05
# @Blog    : https://blog.zbmain.com
import torch


# devices
class devices():
    torch = torch.device(torch.cuda.is_available() and 'cuda' or 'cpu')
    '''torch方式'''
    tensorflow = 'GPU:0'
    '''tensorflow方式'''

# others



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : zb
# @Time    : 2020/11/18 22:27
# @Blog    : https://blog.zbmain.com


def write(char,filepath = 'tmp.txt',mode='w'):
    with open(filepath,mode) as file:
            file.write(str(char)+"\n")




#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : zb
# @Time    : 2020/11/18 10:03
# @Blog    : https://blog.zbmain.com

from __future__ import absolute_import, division, print_function, unicode_literals
from utils import time
import tensorflow as tf
import os


class GRU(object):
    '''
    GRU生成模式
    '''
    def __init__(self, vocab_size, embedding_dim, rnn_units, batch_size, buffer_size=10000,
                 checkpoint_dir='./training_checkpoints'):
        '''
        创建模型
        :param vocab_size: 词汇数，所有特征的数量
        :param embedding_dim: 词嵌入维度
        :param rnn_units: 隐藏层节点数
        :param batch_size: 批次
        :param dataset: 数据
        :param buffer_size: 数据缓存区大小
        '''
        self.vocab_size = vocab_size
        self.embedding_dim = embedding_dim
        self.rnn_units = rnn_units
        self.batch_size = batch_size
        self.buffer_size = buffer_size
        # 默认
        self.checkpoint_dir = checkpoint_dir
        self.checkpoint_prefix = os.path.join(self.checkpoint_dir, 'ckpt_{epoch}')

    def loss(self, labels, logits):
        return tf.keras.losses.sparse_categorical_crossentropy(labels, logits, from_logits=True)

    def __call__(self, dataset):
        self.dataset = dataset
        self.optimizer = tf.keras.optimizers.Adam()
        self.model = self.build_model()
        return self.model

    def build_model(self, vocab_size='', embedding_dim='', rnn_units='', batch_size=0):
        """构建模型并返回"""
        # vocab_size 不设置则拿初始化值
        vocab_size = vocab_size or self.vocab_size
        embedding_dim = embedding_dim or self.embedding_dim
        rnn_units = rnn_units or self.rnn_units
        batch_size = batch_size or self.batch_size

        model = tf.keras.Sequential([
            # embbeding层
            tf.keras.layers.Embedding(vocab_size, embedding_dim, batch_input_shape=[batch_size, None]),
            # GRU模型
            tf.keras.layers.GRU(rnn_units, return_sequences=True, stateful=True,
                                recurrent_initializer='glorot_uniform'),
            # 线性层调整输出维度
            tf.keras.layers.Dense(vocab_size)
        ])
        return model

    # 定义损失函数
    def loss(labels, logits):
        """返回损失函数对象"""
        return tf.keras.losses.sparse_categorical_crossentropy(labels, logits, from_logits=True)


    def train_step(self, inp, target):
        with tf.GradientTape() as tape:
            predictions = self.model(inp)
            loss = tf.reduce_mean(
                tf.keras.losses.sparse_categorical_crossentropy(target, predictions, from_logits=True))

        grads = tape.gradient(loss, self.model.trainable_variables)
        self.optimizer.apply_gradients(zip(grads, self.model.trainable_variables))
        return loss

    def train(self, epochs: int = 10):
        for epoch in range(epochs):
            start = time.time()
            hidden = self.model.reset_states()
            for (batch_n, (inp, target)) in enumerate(self.dataset):
                loss = self.train_step(inp, target)
                if batch_n % 100 == 0:
                    template = 'Epoch {} Batch {} Loss {}'
                    print(template.format(epoch + 1, batch_n, loss))
            if (epoch + 1) % 5 == 0:
                self.model.save_weights(self.checkpoint_prefix.format(epoch=epoch + 1))
            print('Epoch {} Loss {:.4f}'.format(epoch + 1, loss))
            print('Time taken for 1 epoch {} sec\n'.format(time.time() - start))
        self.model.save_weights(self.checkpoint_prefix.format(epoch=epoch + 1))  # 保存最后一次
        return self.batch_size

    def loadModel(self, vocab_size='', embedding_dim='', rnn_units='', batch_size=0):
        vocab_size = vocab_size or self.vocab_size
        embedding_dim = embedding_dim or self.embedding_dim
        rnn_units = rnn_units or self.rnn_units
        batch_size = batch_size or self.batch_size
        print(batch_size)
        self.model = self.build_model(vocab_size, embedding_dim, rnn_units, batch_size)
        # 加载model
        self.model.load_weights(tf.train.latest_checkpoint(checkpoint_dir=self.checkpoint_dir))
        return self.model



from numpy import argmax


class OneHot(object):
    '''
    onehot生成器
    编码：call() || encode()
    解码：decode()
    '''

    def __init__(self):
        # 总类别特征列表
        self.__class_lst = []
        # 编码映射表
        self.__char_to_num = []
        # 源码映射表
        self.__num_to_char = []
        # onehot编码列表
        self.__onehot_encoded = []

    def __call__(self, sourceList:list, classList:list=None):
        '''
        列表 转 onehot编码
        :param sourceList: 源列表
        :param classList: 源列表总特征表。缺省：None（则等于源列表）
        :return:onehot编码列表
        '''
        return self.encode(sourceList, classList)

    def encode(self, sourceList:list, classList:list=None):
        '''
        列表 转 onehot编码（与call方法等价）
        :param sourceList: 源列表
        :param classList: 源列表总特征表。缺省：None（None则为源列表）
        :return:onehot编码列表
        '''
        self.__class_lst = classList or sourceList #没有指定总类型表,则当前列表为总类型
        self.__char_to_num = dict((c, n) for n, c in enumerate(self.__class_lst))
        self.__num_to_char = dict((n, c) for n, c in enumerate(self.__class_lst))
        integer_encoded = [self.__char_to_num[char] for char in sourceList]
        # onehot编码数组
        self.__onehot_encoded = []
        for value in integer_encoded:
            letter = [0 for _ in range(len(self.__class_lst))]
            letter[value] = 1
            self.__onehot_encoded.append(letter)
        return self.__onehot_encoded

    def decode(self, onehotNode:list):
        '''
        onehot编码元素 转 源列表元素
        :param onehotNode: onehot编码返回的元素
        :return:源列表元素
        :example: decode([1,0,0])
        '''
        return self.__num_to_char[argmax(onehotNode)]

    def getNodeOneHot(self, char:str):
        '''
        源列表元素 获取 onehot编码元素
        :param char: 编码源元素
        :return: 该元素的onehot编码
        '''
        return self.__onehot_encoded[self.__char_to_num[char]]

    @property
    def onehotCode(self):
        '''获取onehot码'''
        return self.__onehot_encoded


if __name__ == "__main__":
    onehot = OneHot()
    source = ['a', 'b', 'c', 'd']
    onehot_list = onehot(source)
    print(onehot_list)
    print(onehot.getNodeOneHot(source[1]))
    print(onehot.decode(onehot_list[1]))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : zb
# @Time    : 2020/11/18 13:20
# @Blog    : https://blog.zbmain.com
import pandas as pd


def __merge(datas: list):
    '''添加多个数据文件时使用'''
    with open('all_datas.txt', 'w', encoding='utf-8') as outfile:
        for file in datas:
            with open(file, 'r') as infile:
                outfile.write(infile.read())
    with open('all_datas.txt', 'r', encoding='utf-8') as f:
        data = f.read()
    return data


def read_txt(data):
    if isinstance(data, list):
        data = __merge(data)
    elif isinstance(data, str):
        with open(data, 'r', encoding='utf-8') as f:
            data = f.read()
    else:
        data = ''

    return data


def dataframe():
    df = pd.read_csv('all_datas.txt', sep='\n', names=['w'])
    df["w_length"] = list(map(lambda x: len(x), df["w"]))
    return df


if __name__ == '__main__':
    data = read_txt(['./jay_chou.txt'])
    print(len(data))
    print(dataframe())



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : zb
# @Time    : 2020/11/17 23:21
# @Blog    : https://blog.zbmain.com

import platform
import os

sys_ = platform.system().lower()
__common_data_path = sys_ == 'linux' and '/home/common/' or sys_ == 'darwin' and '/Users/common/' or 'C:\\Users/common/'
'''
    Linux|Mac|Win 运行环境下数据文件绝对路径地址
    [Linux|Mac|Win]
    .__join()
'''


def __join(filepath):
    '''
    绝对数据路径拼接

    :param filepath:
    :return: /common/ [filepath]
    '''
    return os.path.join(__common_data_path, filepath)


if __name__ == '__main__':
    print(__join('main.py'))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : zb
# @Time    : 2020/11/18 16:09
# @Blog    : https://blog.zbmain.com
import re

# 常用的类型
NUM = '0-9'
LETTERS = 'a-zA-Z'
NORMAL_CHAR = '!,.?！，。？'
SPECIAL_CHAR = '’"#$%&\'()*+-/;；<=>@★☆〇〖〗、【】＜＞《》“”‘’\\[\\\\]^_`{|}~'
CN_Unicode = '\u4E00-\u9FA5'#中文
CN_Unicdoe2 = '\u4E00-\u9FEF'#中文2


def clearNull(char: str):
    '''防止爬虫抓到特殊隐藏字符'''
    return re.sub(
        '[\001\002\003\004\005\006\007\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a]+',
        '', char)


def replaces(char: str, *regs, end=''):
    ccc = '[' + ''.join(regs) + ']'
    return re.sub(ccc, end, char)


if __name__ == "__main__":
    print(clearNull('123 abc 你好，世界！'))
    print(replaces('123 abc\你好\\世界！', NUM,LETTERS, NORMAL_CHAR, SPECIAL_CHAR, end='X'))
    print(replaces('123 @abc\你好，\n\世界！', '^',CN_Unicode,',，!！\n\0x0D\0x0A\u3000\u0020', end=''))



import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zbmain",
    version="0.0.6",
    author="zhaobin",
    author_email="zbmain@qq.com",
    description="public python library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/zbmain/py_pub",
    packages=setuptools.find_packages(),
    classifiers=[
         # 该软件包仅与Python3兼容
        "Programming Language :: Python :: 3",
        # 根据MIT许可证开源
        "License :: OSI Approved :: MIT License",
        # 与操作系统无关
        "Operating System :: OS Independent",
    ],
    #.egg
    # zip_safe = False
)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Author  : zb
# @Time    : 2020/11/27 16:43
# @Blog    : https://blog.zbmain.com
import argparse

parser = argparse.ArgumentParser()
# ----set argument----
parser.add_argument("-t","--train", action='store_true', default=False, help="Whether to run training.")
# ----set end----
args = parser.parse_args()